@@ -22,6 +22,8 @@
  *
  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -30,6 +32,7 @@
 #include <glib-object.h>
 
 #include <telepathy-glib/telepathy-glib.h>
+#include <telepathy-glib/telepathy-glib-dbus.h>
 
 static gchar *app_name;
 static GMainLoop *main_loop;
@@ -42,6 +45,8 @@ show_help (gchar * err)
 
     printf ("Usage:\n"
 	    "    %1$s list\n"
+	    "    %1$s summary\n"
+	    "    %1$s dump\n"
 	    "    %1$s add <manager>/<protocol> <display name> [<param> ...]\n"
 	    "    %1$s update <account name> [<param>|clear:key] ...\n"
 	    "    %1$s display <account name> <display name>\n"
@@ -135,14 +140,6 @@ static char *ensure_prefix (char const *string)
     return g_strdup_printf ("%s%s", TP_ACCOUNT_OBJECT_PATH_BASE, string);
 }
 
-static const char *skip_prefix (char const *string)
-{
-    const char *prefixed = strip_prefix (string, TP_ACCOUNT_OBJECT_PATH_BASE);
-    if (prefixed != NULL)
-	return prefixed;
-    return string;
-}
-
 static void
 _g_value_free (gpointer data)
 {
@@ -321,6 +318,20 @@ show_presence (gchar const *what, struct presence *presence)
     presence->type, presence->message);
 }
 
+static int
+show_uri_schemes (const gchar * const *schemes)
+{
+  int result;
+  gchar *tmp;
+
+  tmp = g_strjoinv (", ", (gchar **) schemes);
+
+  result = printf ("%12s: %s\n", "URIScheme", tmp);
+
+  g_free (tmp);
+  return result;
+}
+
 static void
 free_presence (struct presence *presence)
 {
@@ -414,6 +425,7 @@ typedef enum {
     GET_PARAM,
     GET_STRING,
     GET_BOOLEAN,
+    GET_PRESENCE,
     GET_PRESENCE_TYPE,
     GET_PRESENCE_STATUS,
     GET_PRESENCE_MESSAGE
@@ -463,6 +475,8 @@ getter_list_init(void)
 		    tp_account_get_connect_automatically);
     getter_list_add("NormalizedName", GET_STRING, tp_account_get_normalized_name);
 
+    getter_list_add("AutomaticPresence",
+                    GET_PRESENCE, tp_account_get_automatic_presence);
     getter_list_add("AutomaticPresenceType",
 		    GET_PRESENCE_TYPE, tp_account_get_automatic_presence);
     getter_list_add("AutomaticPresenceStatus",
@@ -495,12 +509,25 @@ getter_by_name(char *name)
 
 /* ====================================================================== */
 
+static gint
+compare_accounts (gconstpointer a,
+                  gconstpointer b)
+{
+    return strcmp (tp_account_get_path_suffix (TP_ACCOUNT (a)),
+                   tp_account_get_path_suffix (TP_ACCOUNT (b)));
+}
+
+static GList *
+dup_valid_accounts_sorted (TpAccountManager *manager)
+{
+    return g_list_sort (tp_account_manager_dup_valid_accounts (manager),
+                        compare_accounts);
+}
+
 static gboolean
 command_list (TpAccountManager *manager)
 {
-    GList *accounts;
-
-    accounts = tp_account_manager_get_valid_accounts (manager);
+    GList *accounts = dup_valid_accounts_sorted (manager);
 
     if (accounts != NULL) {
 	GList *ptr;
@@ -508,15 +535,57 @@ command_list (TpAccountManager *manager)
 	command.common.ret = 0;
 
 	for (ptr = accounts; ptr != NULL; ptr = ptr->next) {
-	    puts (skip_prefix (tp_proxy_get_object_path (ptr->data)));
+	    puts (tp_account_get_path_suffix (ptr->data));
 	}
 
-	g_list_free (accounts);
+	g_list_free_full (accounts, g_object_unref);
     }
 
     return FALSE;                 /* stop mainloop */
 }
 
+static gboolean
+command_summary (TpAccountManager *manager)
+{
+    GList *accounts, *l;
+    guint longest_account = 0;
+
+    accounts = tp_account_manager_dup_valid_accounts (manager);
+    if (accounts == NULL) {
+        return FALSE;
+    }
+    command.common.ret = 0;
+
+    for (l = accounts; l != NULL; l = l->next) {
+        TpAccount *account = TP_ACCOUNT (l->data);
+
+        longest_account = MAX (longest_account,
+            strlen (tp_account_get_path_suffix (account)));
+    }
+
+    /* The -6 is so we can line up the "Enabled" header to have the ticks and
+     * crosses below the 7th and final character. We're only guaranteed
+     * longest_account ≥ 5 in theory (a/b/c is the shortest legal suffix) but
+     * in practice it's always going to be ≥ 7.
+     */
+    g_return_val_if_fail (longest_account >= 7, FALSE);
+    printf ("%-*s %s %s\n", longest_account - 6, "Account", "Enabled", "Requested");
+    printf ("%-*s %s %s\n", longest_account - 6, "=======", "=======", "=========");
+
+    for (l = accounts; l != NULL; l = l->next) {
+        TpAccount *account = TP_ACCOUNT (l->data);
+        gchar *status;
+
+        tp_account_get_requested_presence (account, &status, NULL);
+        printf ("%-*s %s %s\n",
+            longest_account, tp_account_get_path_suffix (account),
+            tp_account_is_enabled (account) ? "✓" : "☐",
+            status);
+    }
+
+    g_list_free_full (accounts, g_object_unref);
+    return FALSE; /* stop mainloop */
+}
 
 static void
 callback_for_create_account (GObject *source,
@@ -531,7 +600,7 @@ callback_for_create_account (GObject *source,
 
     if (account != NULL) {
 	command.common.ret = 0;
-	puts (skip_prefix (tp_proxy_get_object_path (account)));
+	puts (tp_account_get_path_suffix (account));
     }
     else {
 	fprintf (stderr, "%s %s: %s\n", app_name, command.common.name,
@@ -574,7 +643,7 @@ callback_for_update_parameters (GObject *source,
             printf ("  %s\n", r);
             printf ("run:\n");
             printf ("  %s reconnect %s\n", app_name,
-                    skip_prefix (command.common.account));
+                    tp_account_get_path_suffix (account));
             g_free (r);
             g_strfreev (reconnect_required);
         }
@@ -617,18 +686,48 @@ command_remove (TpAccount *account)
     return TRUE;
 }
 
+static gchar *
+dup_storage_restrictions (TpAccount *account)
+{
+  TpStorageRestrictionFlags flags;
+  GPtrArray *tmp;
+  gchar *result;
+
+  flags = tp_account_get_storage_restrictions (account);
+  if (flags == 0)
+    return g_strdup ("(none)");
+
+  tmp = g_ptr_array_new ();
+
+  if (flags & TP_STORAGE_RESTRICTION_FLAG_CANNOT_SET_PARAMETERS)
+    g_ptr_array_add (tmp, "Cannot_Set_Parameters");
+  if (flags & TP_STORAGE_RESTRICTION_FLAG_CANNOT_SET_ENABLED)
+    g_ptr_array_add (tmp, "Cannot_Set_Enabled");
+  if (flags & TP_STORAGE_RESTRICTION_FLAG_CANNOT_SET_PRESENCE)
+    g_ptr_array_add (tmp, "Cannot_Set_Presence");
+  if (flags & TP_STORAGE_RESTRICTION_FLAG_CANNOT_SET_SERVICE)
+    g_ptr_array_add (tmp, "Cannot_Set_Service");
+
+  g_ptr_array_add (tmp, NULL);
+
+  result = g_strjoinv (", ", (gchar **) tmp->pdata);
+
+  g_ptr_array_unref (tmp);
+  return result;
+}
+
 static gboolean
 command_show (TpAccount *account)
 {
-    gchar const *name;
     const GHashTable *parameters;
     GHashTableIter i[1];
     gpointer keyp, valuep;
     struct presence automatic, current, requested;
+    const gchar * const *schemes;
+    const gchar *storage_provider;
+    const gchar * const *supersedes;
 
-    name = skip_prefix (command.common.account);
-
-    show ("Account", name);
+    show ("Account", tp_account_get_path_suffix (account));
     show ("Display Name", tp_account_get_display_name (account));
     show ("Normalized", tp_account_get_normalized_name (account));
     show ("Enabled", tp_account_is_enabled (account) ? "enabled" : "disabled");
@@ -663,6 +762,51 @@ command_show (TpAccount *account)
     show ("Changing",
         tp_account_get_changing_presence (account) ? "yes" : "no");
 
+    schemes = tp_account_get_uri_schemes (account);
+    if (schemes != NULL && schemes[0] != NULL)
+      {
+        puts ("");
+        puts ("Addressing:");
+
+        show_uri_schemes (schemes);
+      }
+
+    storage_provider = tp_account_get_storage_provider (account);
+    if (!tp_str_empty (storage_provider))
+      {
+        GVariant *storage_identifier;
+        gchar *storage_restrictions;
+
+        puts ("");
+        puts ("Storage:");
+        show ("Provider", storage_provider);
+
+        storage_identifier = tp_account_dup_storage_identifier_variant (
+            account);
+        if (storage_identifier != NULL)
+          {
+            gchar *tmp = g_variant_print (storage_identifier, TRUE);
+
+            show ("Identifier", tmp);
+
+            g_free (tmp);
+            g_variant_unref (storage_identifier);
+          }
+
+        storage_restrictions = dup_storage_restrictions (account);
+        show ("Restrictions", storage_restrictions);
+        g_free (storage_restrictions);
+      }
+
+    supersedes = tp_account_get_supersedes (account);
+    if (supersedes != NULL && supersedes[0] != NULL)
+      {
+        puts ("");
+        puts ("Supersedes:");
+        for (; *supersedes != NULL; supersedes++)
+          printf ("  %s\n", *supersedes + strlen (TP_ACCOUNT_OBJECT_PATH_BASE));
+      }
+
     puts ("");
     parameters = tp_account_get_parameters (account);
 
@@ -676,6 +820,30 @@ command_show (TpAccount *account)
     return FALSE;
 }
 
+static gboolean
+command_dump (TpAccountManager *manager)
+{
+    GList *accounts, *l;
+
+    accounts = tp_account_manager_dup_valid_accounts (manager);
+    if (accounts == NULL) {
+        return FALSE;
+    }
+    command.common.ret = 0;
+
+    for (l = accounts; l != NULL; l = l->next) {
+        TpAccount *account = TP_ACCOUNT (l->data);
+
+        command_show (account);
+
+        if (l->next != NULL)
+          printf ("\n------------------------------------------------------------\n\n");
+    }
+
+    g_list_free_full (accounts, g_object_unref);
+    return FALSE; /* stop mainloop */
+}
+
 static gboolean
 command_connection (TpAccount *account)
 {
@@ -698,7 +866,7 @@ command_connection (TpAccount *account)
     }
     else {
 	fprintf(stderr, "%s: no connection\n",
-		skip_prefix (command.common.account));
+		tp_account_get_path_suffix (account));
     }
 
     return FALSE;
@@ -729,6 +897,16 @@ command_get (TpAccount *account)
 	    else if (getter->type == GET_BOOLEAN) {
 		puts(getboolean(account) ? "true" : "false");
 	    }
+            else if (getter->type == GET_PRESENCE)
+            {
+                struct presence presence;
+
+                presence.type = getpresence(account, &presence.status,
+                    &presence.message);
+                printf ("(%u, \"%s\", \"%s\")\n", presence.type,
+                    presence.status, presence.message);
+                free_presence (&presence);
+            }
 	    else if (getter->type == GET_PRESENCE_TYPE ||
 		     getter->type == GET_PRESENCE_STATUS ||
 		     getter->type == GET_PRESENCE_MESSAGE) {
@@ -952,6 +1130,22 @@ parse (int argc, char **argv)
 
 	command.ready.manager = command_list;
     }
+    else if (strcmp (argv[1], "summary") == 0)
+    {
+        /* List accounts */
+        if (argc != 2)
+            show_help ("Invalid summary command.");
+
+        command.ready.manager = command_summary;
+    }
+    else if (strcmp (argv[1], "dump") == 0)
+    {
+        /* Dump all accounts */
+        if (argc != 2)
+            show_help ("Invalid dump command.");
+
+        command.ready.manager = command_dump;
+    }
     else if (strcmp  (argv[1], "remove") == 0
 	     || strcmp (argv[1], "delete") == 0)
     {
@@ -1202,20 +1396,21 @@ void manager_ready (GObject *manager,
 }
 
 static
-void account_ready (GObject *account,
+void account_ready (GObject *source,
 		    GAsyncResult *res,
 		    gpointer user_data)
 {
+    TpAccount *account = TP_ACCOUNT (source);
     GError *error = NULL;
 
     if (!tp_proxy_prepare_finish (account, res, &error)) {
         fprintf (stderr, "%s: couldn't load account '%s': %s\n", app_name,
-            skip_prefix (command.common.account), error->message);
+            tp_account_get_path_suffix (account), error->message);
         fprintf (stderr, "Try '%s list' to list known accounts.\n", app_name);
         g_error_free (error);
     }
     else {
-        if (command.ready.account (TP_ACCOUNT (account)))
+        if (command.ready.account (account))
             return;
     }
 
@@ -1228,7 +1423,10 @@ main (int argc, char **argv)
     TpAccountManager *am = NULL;
     TpAccount *a = NULL;
     TpDBusDaemon *dbus = NULL;
+    TpSimpleClientFactory *client_factory = NULL;
     GError *error = NULL;
+    const GQuark features[] = { TP_ACCOUNT_FEATURE_CORE,
+        TP_ACCOUNT_FEATURE_ADDRESSING, TP_ACCOUNT_FEATURE_STORAGE, 0 };
 
     g_type_init ();
 
@@ -1244,14 +1442,23 @@ main (int argc, char **argv)
             app_name, command.common.name, error->message);
         goto out;
     }
+    client_factory = tp_simple_client_factory_new (dbus);
 
     if (command.common.account == NULL) {
+        TpSimpleClientFactory *factory;
+
         am = tp_account_manager_new (dbus);
+        factory = tp_proxy_get_factory (am);
+
+        tp_simple_client_factory_add_account_features (factory, features);
+
         tp_proxy_prepare_async (am, NULL, manager_ready, NULL);
     }
     else {
+
 	command.common.account = ensure_prefix (command.common.account);
-	a = tp_account_new (dbus, command.common.account, &error);
+        a = tp_simple_client_factory_ensure_account (client_factory,
+            command.common.account, NULL, &error);
 
 	if (error != NULL)
 	{
@@ -1261,7 +1468,7 @@ main (int argc, char **argv)
 	    goto out;
 	}
 
-	tp_proxy_prepare_async (a, NULL, account_ready, NULL);
+	tp_proxy_prepare_async (a, features, account_ready, NULL);
     }
 
     main_loop = g_main_loop_new (NULL, FALSE);
@@ -1269,6 +1476,7 @@ main (int argc, char **argv)
 
 out:
     g_clear_error (&error);
+    tp_clear_object (&client_factory);
     tp_clear_object (&dbus);
     tp_clear_object (&am);
     tp_clear_object (&a);
